//We always have to include the library
#include <OSCBoards.h>
#include <OSCBundle.h>
#include <OSCData.h>
#include <OSCMatch.h>
#include <OSCMessage.h>
#include <OSCTiming.h>
#include <SLIPEncodedSerial.h>
#include <SLIPEncodedUSBSerial.h>

#ifdef BOARD_HAS_USB_SERIAL
#include <SLIPEncodedUSBSerial.h>
SLIPEncodedUSBSerial SLIPSerial( thisBoardsSerialUSB );
#else
#include <SLIPEncodedSerial.h>
SLIPEncodedSerial SLIPSerial(Serial);
#endif

#include <Adafruit_NeoPixel.h>

#define PIN1      4
#define PIN2      6
#define N_LEDS 41

Adafruit_NeoPixel strip1 = Adafruit_NeoPixel(N_LEDS, PIN1, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel strip2 = Adafruit_NeoPixel(N_LEDS, PIN2, NEO_GRB + NEO_KHZ800);

const int GSR1 = A0;
int threshold1 = 0;
int sensorValue1;

const int GSR2 = A2;
int threshold2 = 0;
int sensorValue2;

int speed1;
int speed2;
uint32_t color;

unsigned long gsr_ts;
const int gsr_interval = 1000;

void setup() {
  strip1.begin();
  strip1.show(); // Initialize all pixels to 'off'
  strip2.begin();
  strip2.show(); // Initialize all pixels to 'off'

  // initialize serial communications at 9600 bps:
  long sum1 = 0;
  Serial.begin(9600);

  for (int i = 0; i < 500; i++)
  {
    sensorValue1 = analogRead(GSR1);
    sum1 += sensorValue1;
    delay(5);
  }
  threshold1 = sum1 / 500;

  // initialize serial communications at 9600 bps:

  long sum2 = 0;
  Serial.begin(9600);

  for (int x = 0; x < 500; x++)
  {
    sensorValue2 = analogRead(GSR2);
    sum2 += sensorValue2;
    delay(5);
  }
  threshold2 = sum2 / 500;


  gsr_ts = millis();
}

void showNeoPixel1(OSCMessage &msg) {
  if (msg.isInt(0)) {
    if ((msg.getInt(0) >= 0)) {
      speed1 = msg.getInt(0);

      if (speed1 == 40 || speed1 == 30) {
        chase1(strip1.Color(0, 255, 0)); // Green
      }
      if (speed1 == 20 || speed1 == 10) {
        chase1(strip1.Color(255, 255, 0)); // Yellow
      }
      if (speed1 == 5) {
        chase1(strip1.Color(255, 0, 0)); // Red
      }
    }
  }
}

void showNeoPixel2(OSCMessage &msg) {
  if (msg.isInt(0)) {
    if ((msg.getInt(0) >= 0)) {
      speed2 = msg.getInt(0);

      if (speed2 == 40 || speed2 == 30) {
        chase2(strip2.Color(0, 255, 0)); // Green
      }
      if (speed2 == 20 || speed2 == 10) {
        chase2(strip2.Color(255, 255, 0)); // Yellow
      }
      if (speed2 == 5) {
        chase2(strip2.Color(255, 0, 0)); // Red
      }
    }
  }
}

static void chase1(uint32_t color) {
  for (uint16_t i = 0; i < strip1.numPixels() + 6; i++) {
    strip1.setPixelColor(i  , color); // Draw new pixel
    strip1.setPixelColor(i - 6, 0); // Erase pixel a few steps back
    strip1.show();
    delay(25);
  }
}

static void chase2(uint32_t color) {
  for (uint16_t i = 0; i < strip2.numPixels() + 6; i++) {
    strip2.setPixelColor(i  , color); // Draw new pixel
    strip2.setPixelColor(i - 6, 0); // Erase pixel a few steps back
    strip2.show();
    delay(25);
  }
}

void loop() {
  if ( millis() - gsr_ts >= gsr_interval ) {  // if gsr_interval(ms) has passed
    gsr_ts = millis();  // gsr_ts = current time

    OSCBundle bndout;

    bndout.add("/gsr1").add(sensorValue1);
    SLIPSerial.beginPacket(); // mark the beginning of the OSC Packet
    bndout.send(SLIPSerial);
    SLIPSerial.endPacket();

    bndout.add("/gsr2").add(sensorValue2);
    SLIPSerial.beginPacket(); // mark the beginning of the OSC Packet
    bndout.send(SLIPSerial);
    SLIPSerial.endPacket();


    // read the analog in value:
    int temp1;
    sensorValue1 = analogRead(GSR1);
    temp1 = sensorValue1 - threshold1;

    int temp2;
    sensorValue2 = analogRead(GSR2);
    temp2 = sensorValue2 - threshold2;
  }

  // delay(1000);


  OSCBundle bundleIN;
  int size;

  if (SLIPSerial.available()) {
    while (!SLIPSerial.endofPacket())
      if ( (size = SLIPSerial.available()) > 0)
      {
        while (size--)
          bundleIN.fill(SLIPSerial.read());
      }

    if (!bundleIN.hasError()) {
      bundleIN.dispatch("/speed1", showNeoPixel1);
      bundleIN.dispatch("/speed2", showNeoPixel2);
    }
  }

}
